import BBRest from './index.js';

describe('Testing BBRest instance...', function() {
  it('should fail if plugin is not defined', function() {
    expect(BBRest).toThrowError(Error);
  });

  it('should target test-portal', function() {
    const bbrest = BBRest({
      portal: 'test-portal',
      plugin: {},
    });
    expect(bbrest.config.portal).toBe('test-portal');
  });
});

describe('Testing BBReq instances...', function() {
  let bbrest;
  let requestConfig;
  let uri;

  beforeEach(function() {
    bbrest = BBRest({
      portal: 'test-portal',
      plugin: {
        getRequestBody: function() {
          return Promise.resolve('<yo/>');
        },
        request: function(obj) {
          requestConfig = obj;
          return Promise.resolve({
            status: 200,
            statusText: 'OK',
            headers: {},
            body: requestConfig.body,
          });
        },
        response: (config, toSend, response) => Promise.resolve(config.body)
      }
    });
    uri = `${bbrest.config.scheme}://${bbrest.config.host}:${bbrest.config.port}/${bbrest.config.context}/`;
  });


  describe('Testing server endpoints...', function() {
    let serverReq;

    beforeEach(function() {
      serverReq = bbrest.server();
    });

    it('should return BBReq instance, server command', function() {
      expect(serverReq.command).toBe('server');
    });

    it('should contain config', function() {
      expect(bbrest.config).toEqual(serverReq.config);
    });

    it('should have proper Content-type header application/xml', function() {
      expect(serverReq.headers['Content-Type']).toBe('application/xml');
    });

    it('should set query', function() {
      const o = {
        num: 123,
        float: 1.23,
        str: 'hello world',
        bool: false
      };
      serverReq.query(o);
      expect(serverReq.qs).toBe(o);
    });

    it('should set get', function() {
      serverReq.get()
      expect(requestConfig.method).toBe('GET');
    });

    it('should set post', function() {
      serverReq.post();
      expect(requestConfig.method).toBe('POST');
    });

    it('should set put', function() {
      serverReq.put();
      expect(requestConfig.method).toBe('PUT');
    });

    it('should set delete', function() {
      serverReq.delete();
      expect(requestConfig.method).toBe('DELETE');
    });

    it('should not treat js object as file path', function() {
      serverReq.post({foo: 'boo'});
      expect(requestConfig.body).toBe('<foo>boo</foo>');
    });

    it('should not treat xml as file path', function() {
      const pl = '<xml attr="boo"></xml>';
      serverReq.post(pl);
      expect(requestConfig.body).toBe(pl);
    });

    it('should throw error if payload is wrong type', function() {
      expect(function() {
        serverReq.post(4);
      }).toThrowError(Error);
    });

    it('should treat non empty string as file path', function(done) {
      serverReq.post('./my.xml')
        .then(function(response) {
          expect(requestConfig.body).toBe('<yo/>');
          done();
        })
        .catch(function(err) {
          console.log(err.message);
        });
    });

    it('should convert jxon payload to xml', function() {
      serverReq.post({foo: 'boo'});
      expect(requestConfig.body).toBe('<foo>boo</foo>');
    });

    it('server.get url', function() {
      serverReq.get();
      expect(requestConfig.url).toBe(uri + 'portals');
    });

    it('server.post url', function() {
      serverReq.post();
      expect(requestConfig.url).toBe(uri + 'portals');
    });

    it('server.put url', function() {
      serverReq.put();
      expect(requestConfig.url).toBe(uri + 'portals');
    });
  });

  describe('Testing portal endpoints...', function() {
    let portalReq;

    beforeEach(function() {
      portalReq = bbrest.portal();
    });

    it('portal.get url', function() {
      portalReq.get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal.xml');

    });

    it('portal.rights.get url', function() {
      portalReq.rights().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/rights');
    });

    it('portal.rights.put url', function() {
      portalReq.rights().put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/rights');
    });

    it('portal.tag.get url', function() {
      portalReq.tag().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/tags');
    });

    it('portal.tag.post url', function() {
      portalReq.tag().post();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/tags');
    });

    it('portal.tag(name).delete url', function() {
      portalReq.tag('my-tag').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/tags/my-tag');
    });

    it('portal.put url', function() {
      portalReq.put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal');
    });

    it('portal.delete url', function() {
      portalReq.delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal');
    });
  });

  describe('Testing catalog endpoints...', function() {
    let catalogReq;

    beforeEach(function() {
      catalogReq = bbrest.catalog();
    });

    it('catalog.get url', function() {
      catalogReq.get();
      expect(requestConfig.url).toBe(uri + 'catalog');
    });

    it('catalog.post url', function() {
      catalogReq.post();
      expect(requestConfig.url).toBe(uri + 'catalog');
    });

    it('catalog.put url', function() {
      catalogReq.put();
      expect(requestConfig.url).toBe(uri + 'catalog');
    });

    it('catalog.delete(batch) url', function() {
      catalogReq.delete('<>');
      expect(requestConfig.url).toBe(uri + 'delete/catalog');
    });

    it('catalog(item).get url', function() {
      bbrest.catalog('my-item').get();
      expect(requestConfig.url).toBe(uri + 'catalog/my-item');
    });

    it('catalog(item).delete url', function() {
      bbrest.catalog('my-item').delete();
      expect(requestConfig.url).toBe(uri + 'catalog/my-item');
    });
  });

  describe('Testing portal catalog endpoints...', function() {
    let catalogReq;

    beforeEach(function() {
      catalogReq = bbrest.portalCatalog();
    });

    it('catalog.get url', function() {
      catalogReq.get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog');
    });

    it('catalog.post url', function() {
      catalogReq.post();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog');
    });

    it('catalog.put url', function() {
      catalogReq.put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog');
    });

    it('catalog.delete(batch) url', function() {
      catalogReq.delete('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/delete/catalog');
    });

    it('catalog(item).get url', function() {
      bbrest.portalCatalog('my-item').get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog/my-item.xml');
    });

    it('catalog(item).delete url', function() {
      bbrest.portalCatalog('my-item').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog/my-item');
    });
  });

  describe('Testing page endpoints...', function() {
    it('page.get url', function() {
      bbrest.page().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages');
    });

    it('page.post url', function() {
      bbrest.page().post('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages');
    });

    it('page.put url', function() {
      bbrest.page().put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages');
    });

    it('page(name).get url', function() {
      bbrest.page('my-page').get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages/my-page.xml');
    });

    it('page(name).put url', function() {
      bbrest.page('my-page').put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages/my-page');
    });

    it('page(name).delete url', function() {
      bbrest.page('my-page').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages/my-page');
    });

    it('page(name).rights.get url', function() {
      bbrest.page('my-page').rights().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages/my-page/rights');
    });

    it('page(name).rights.put url', function() {
      bbrest.page('my-page').rights().put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/pages/my-page/rights');
    });
  });

  describe('Testing container endpoints...', function() {
    it('container.get url', function() {
      bbrest.container().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers');
    });

    it('container.post url', function() {
      bbrest.container().post('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers');
    });

    it('container.put url', function() {
      bbrest.container().put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers');
    });

    it('container(name).get url', function() {
      bbrest.container('my-container').get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers/my-container.xml');
    });

    it('container(name).put url', function() {
      bbrest.container('my-container').put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers/my-container');
    });

    it('container(name).delete url', function() {
      bbrest.container('my-container').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers/my-container');
    });

    it('container(name).rights.get url', function() {
      bbrest.container('my-container').rights().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers/my-container/rights');
    });

    it('container(name).rights.put url', function() {
      bbrest.container('my-container').rights().put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers/my-container/rights');
    });
  });

  describe('Testing widget endpoints...', function() {
    it('widget.get url', function() {
      bbrest.widget().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets');
    });

    it('widget.post url', function() {
      bbrest.widget().post('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets');
    });

    it('widget.put url', function() {
      bbrest.widget().put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets');
    });

    it('widget(name).get url', function() {
      bbrest.widget('my-widget').get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets/my-widget.xml');
    });

    it('widget(name).put url', function() {
      bbrest.widget('my-widget').put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets/my-widget');
    });

    it('widget(name).delete url', function() {
      bbrest.widget('my-widget').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets/my-widget');
    });

    it('widget(name).rights.get url', function() {
      bbrest.widget('my-widget').rights().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets/my-widget/rights');
    });

    it('widget(name).rights.put url', function() {
      bbrest.widget('my-widget').rights().put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets/my-widget/rights');
    });
  });

  describe('Testing link endpoints...', function() {
    it('link.get url', function() {
      bbrest.link().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links');
    });

    it('link.post url', function() {
      bbrest.link().post('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links');
    });

    it('link.put url', function() {
      bbrest.link().put();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links');
    });

    it('link(name).get url', function() {
      bbrest.link('my-link').get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links/my-link.xml');
    });

    it('link(name).put url', function() {
      bbrest.link('my-link').put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links/my-link');
    });

    it('link(name).delete url', function() {
      bbrest.link('my-link').delete();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links/my-link');
    });

    it('link(name).rights.get url', function() {
      bbrest.link('my-link').rights().get();
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links/my-link/rights');
    });

    it('link(name).rights.put url', function() {
      bbrest.link('my-link').rights().put('<>');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/links/my-link/rights');
    });
  });

  describe('Testing template endpoints...', function() {
    it('template.get url', function() {
      bbrest.template().get();
      expect(requestConfig.url).toBe(uri + 'templates');
    });

    it('template.post url', function() {
      bbrest.template().post('<>');
      expect(requestConfig.url).toBe(uri + 'templates');
    });

    it('template(name).get url', function() {
      bbrest.template('my-template').get();
      expect(requestConfig.url).toBe(uri + 'templates/my-template');
    });

    it('template(name).put url', function() {
      bbrest.template('my-template').put('<>');
      expect(requestConfig.url).toBe(uri + 'templates/my-template');
    });

    it('template(name).delete url', function() {
      bbrest.template('my-template').delete();
      expect(requestConfig.url).toBe(uri + 'templates/my-template');
    });

    it('template(name).rights.get url', function() {
      bbrest.template('my-template').rights().get();
      expect(requestConfig.url).toBe(uri + 'templates/my-template/rights');
    });

    it('template(name).rights.put url', function() {
      bbrest.template('my-template').rights().put('<>');
      expect(requestConfig.url).toBe(uri + 'templates/my-template/rights');
    });
  });

  describe('Testing user endpoints...', function() {
    it('user.get url', function() {
      bbrest.user().get();
      expect(requestConfig.url).toBe(uri + 'users');
    });

    it('user.post url', function() {
      bbrest.user().post('<>');
      expect(requestConfig.url).toBe(uri + 'users');
    });

    it('user(user).get url', function() {
      bbrest.user('my-user').get();
      expect(requestConfig.url).toBe(uri + 'users/my-user');
    });

    it('user(user).put url', function() {
      bbrest.user('my-user').put('<>');
      expect(requestConfig.url).toBe(uri + 'users/my-user');
    });

    it('user(user).delete url', function() {
      bbrest.user('my-user').delete();
      expect(requestConfig.url).toBe(uri + 'users/my-user');
    });

    it('user.get group url', function() {
      bbrest.user('my-user', true).get();
      expect(requestConfig.url).toBe(uri + 'users/my-user/groups');
    });

    it('user.post group url', function() {
      bbrest.user('my-user', true).post('<>');
      expect(requestConfig.url).toBe(uri + 'users/my-user/groups');
    });

    it('user.delete from group url', function() {
      bbrest.user('my-user', true, 'my-group').delete();
      expect(requestConfig.url).toBe(uri + 'users/my-user/groups/my-group');
    });
  });

  describe('Testing group endpoints...', function() {
    it('group.get url', function() {
      bbrest.group().get();
      expect(requestConfig.url).toBe(uri + 'groups');
    });

    it('group.post url', function() {
      bbrest.group().post('<>');
      expect(requestConfig.url).toBe(uri + 'groups');
    });

    it('group(group).get url', function() {
      bbrest.group('my-group').get();
      expect(requestConfig.url).toBe(uri + 'groups/my-group');
    });

    it('group(group).put url', function() {
      bbrest.group('my-group').put('<>');
      expect(requestConfig.url).toBe(uri + 'groups/my-group');
    });

    it('group(group).delete url', function() {
      bbrest.group('my-group').delete();
      expect(requestConfig.url).toBe(uri + 'groups/my-group');
    });

    it('group.get users url', function() {
      bbrest.group('my-group', true).get();
      expect(requestConfig.url).toBe(uri + 'groups/my-group/users');
    });

    it('group.post users url', function() {
      bbrest.group('my-group', true).post('<>');
      expect(requestConfig.url).toBe(uri + 'groups/my-group/users');
    });

    it('group.delete user url', function() {
      bbrest.group('my-group', true, 'my-user').delete();
      expect(requestConfig.url).toBe(uri + 'groups/my-group/users/my-user');
    });
  });

  describe('Testing audit endpoints...', function() {
    it('auditevents url', function() {
      bbrest.audit().get();
      expect(requestConfig.url).toBe(uri + 'auditevents');
    });

    it('auditmeta url', function() {
      bbrest.audit(true).get();
      expect(requestConfig.url).toBe(uri + 'auditmeta');
    });
  });

  describe('Testing cache endpoints...', function() {
    it('delete any cache url', function() {
      bbrest.cache('my-cache').delete();
      expect(requestConfig.url).toBe(uri + 'caches/my-cache');
    });

    // it('delete all cache url', function() {
    //     bbrest.cache('all').delete();
    //     assert.equal(requestConfig.url, uri + 'caches/my-cache');
    // });
  });

 describe('Testing auto method', function() {
    it('should do server POST', function() {
      const j = {portals: {
        portal: {
          contextItemName: '[BBHOST]'
        }
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('POST');
      expect(requestConfig.url).toBe(uri + 'portals');
    });
    it('should do server PUT', function() {
      const j = {portal: {
        contextItemName: '[BBHOST]'
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('PUT');
      expect(requestConfig.url).toBe(uri + 'portals');
    });
    it('should do server catalog POST', function() {
      const j = {catalog: {
        widget: {
          contextItemName: '[BBHOST]'
        }
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('POST');
      expect(requestConfig.url).toBe(uri + 'catalog');
    });
    it('should do server catalog PUT', function() {
      const j = {catalog: {
        widget: {
          contextItemName: '[BBHOST]'
        }
      }};
      bbrest.auto(j, 'put');
      expect(requestConfig.method).toBe('PUT');
      expect(requestConfig.url).toBe(uri + 'catalog');
    });
    it('should do portal catalog POST', function() {
      const j = {catalog: {
        widget: {
          contextItemName: 'test-portal'
        }
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('POST');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/catalog');
    });
    it('should do container POST', function() {
      const j = {containers: {
        container: {
          contextItemName: 'test-portal'
        }
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('POST');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/containers');
    });
    it('should do widget PUT', function() {
      const j = {widget: {
        contextItemName: 'test-portal'
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('PUT');
      expect(requestConfig.url).toBe(uri + 'portals/test-portal/widgets');
    });
    it('should do user POST', function() {
      const j = {users: {
        user: {
          username: 'myName'
        }
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('POST');
      expect(requestConfig.url).toBe(uri + 'users');
    });
    it('should do user PUT', function() {
      const j = {user: {
        username: 'myName'
      }};
      bbrest.auto(j);
      expect(requestConfig.method).toBe('PUT');
      expect(requestConfig.url).toBe(uri + 'users');
    });
  });

  describe('Testing import...', function() {
    it('import.post url', function() {
      bbrest.import().post('<>');
      expect(requestConfig.url).toBe(uri + 'import/portal');
    });

    it('orchestartor import.file(file).post url', function() {
      bbrest.import().file('my-file').post();
      expect(requestConfig.file).toBe('my-file');
      expect(requestConfig.url).toBe(uri + 'orchestrator/import/upload');
    });
  });

  describe('Testing export...', function() {
    it('export.get url', function() {
      bbrest.export().get();
      expect(requestConfig.url.indexOf(uri + 'export/portal')).toBe(0);
      expect(requestConfig.query.portalName).toBe('test-portal');
    });

    it('orchestrator export.post url', function() {
      bbrest.export().post('<>');
      expect(requestConfig.url.indexOf(uri + 'orchestrator/export/exportrequests')).toBe(0);
    });

    it('orchestrator export(id).file(file).get url', function() {
      bbrest.export('my-id').file('my-file').get();
      expect(requestConfig.file).toBe('my-file');
      expect(requestConfig.url).toBe(uri + 'orchestrator/export/files/my-id');
    });
  });

  describe('Testing import item...', function() {
    it('importItem.post url', function() {
      bbrest.importItem().post();
      expect(requestConfig.url).toBe(uri + 'import/package');
    });

    it('portal importItem.post url', function() {
      bbrest.importItem(true).post();
      expect(requestConfig.url).toBe(uri + 'import/package/test-portal');
    });
  });

  describe('Testing export item...', function() {
    it('exportItem(name).get url', function() {
      bbrest.exportItem('my-item').get();
      expect(requestConfig.url).toBe(uri + 'export/package/my-item');
    });

    it('portal exportItem(name).get url', function() {
      bbrest.exportItem('my-item', true).get();
      expect(requestConfig.url).toBe(uri + 'export/package/test-portal/my-item');
    });
  });

});

