import * as utils from './utils';
import matches2xx from './errors/2xx';
import matches5xx from './errors/5xx';

class CXPError extends Error {
  constructor(message, raw, toSend, config) {
    super(`${message} ${toSend.url}`);
    this.response = raw;
    this.request = toSend;
    this.config = config;
    Error.captureStackTrace(this, this.constructor.name);
  }
}

function getContentType(headers) {
  const typeHeader = headers['content-type'];
  return typeHeader ? typeHeader.split(';')[0] : '';
}

function matchError(matches, headers, body) {
  const contentType = getContentType(headers);

  const match = matches.find((item) => 
    body.indexOf(item.search) > -1 && item.contentType === contentType
  );
  if (match) return match.callback(body);
  return false;
}

/*
 * config - bbrest main config object(scheme, host etc)
 * toSend - object passed to plugin.request(url, method, query, headers...)
 * res - response object(status, statusText, headers, body)
 */
export default function parseResponse(config, toSend, res) {
  let errorMessage;

  if (res.status >= 200 && res.status < 300) {
    if (errorMessage = matchError(matches2xx, res.headers, res.body)) {
      throw new CXPError(errorMessage, res, toSend, config);
    }
    // download
    if (getContentType(res.headers) === 'application/octet-stream') return toSend.file;

    if (config.outputJxon) {
      return utils.stringToJs(res.body);
    }
    return res.body;
  } else if (res.status >= 300 && res.status < 400) {
    const href = config.plugin.urlParse(res.headers.location);
    throw new CXPError(`CXPRedirect: ${href.query.errorMessage}`, res, toSend, config);
  } else if (res.status >= 400 && res.status < 500) {
    throw new CXPError(`${res.status} ${res.statusText}`, res, toSend, config);
  }
  if (errorMessage = matchError(matches5xx, res.headers, res.body)) {
    throw new CXPError(errorMessage, res, toSend, config);
  }
  throw new CXPError(`${res.status} ${res.statusText}`, res, toSend, config);
}
